/*
Copyright (C) 2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.guicomponents;

import java.io.File;
import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.LineUnavailableException;
import org.harctoolbox.IrpMaster.IncompatibleArgumentException;
import org.harctoolbox.IrpMaster.ModulatedIrSequence;
import org.harctoolbox.IrpMaster.Wave;
import org.harctoolbox.harchardware.ir.IrAudioDevice;

public class AudioParametersBean extends javax.swing.JPanel {

    private final static AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
    private final static boolean bigEndian = false;
    private final static boolean divideCarrier = true;
    private final static boolean square = true;
    private boolean verbose = false;
    private int sampleSize = 8;

    /**
     * Creates new form AudioParametersBean
     */
    public AudioParametersBean() {
        initComponents();
    }

    /**
     * @param verbose the verbose to set
     */
    public void setVerbose(boolean verbose) {
        this.verbose = verbose;
    }

    /**
     * @return the sampleSize
     */
    public int getSampleSize() {
        return sampleSize;
    }

    /**
     * @return the bigEndian
     */
    public boolean getBigEndian() {
        return bigEndian;
    }

    /**
     * @return the divideCarrier
     */
    public boolean getDivideCarrier() {
        return divideCarrier;
    }

    /**
     * @return the square
     */
    public boolean getSquare() {
        return square;
    }

    public int getChannels() {
        return Integer.parseInt((String) channelsComboBox.getSelectedItem());
    }

    public int getSampleFrequency() {
        return Integer.parseInt((String) sampleFrequencyComboBox.getSelectedItem());
    }

    public boolean getOmitTrailingGap() {
        return omitTrailingGapCheckBox.isSelected();
    }

    public AudioFormat getAudioFormat() {
        return new AudioFormat(encoding, getSampleFrequency(), getSampleSize(), getChannels(),
                getSampleSize()/8*getChannels(), getSampleFrequency(), getBigEndian());
    }

    public Wave newWave(ModulatedIrSequence irSequence) throws IncompatibleArgumentException {
        return new Wave(irSequence, getAudioFormat(), getOmitTrailingGap(), getSquare(), getDivideCarrier());
    }

    public IrAudioDevice newIrAudioDevice() {
        return new IrAudioDevice(getSampleFrequency(), getChannels(), getOmitTrailingGap(), verbose);
    }

    public void export(File file, ModulatedIrSequence irSequence) throws IncompatibleArgumentException {
        newWave(irSequence).export(file);
    }

    public void play(ModulatedIrSequence irSequence) throws LineUnavailableException, IncompatibleArgumentException, IOException {
        newWave(irSequence).play();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        sampleFrequencyComboBox = new javax.swing.JComboBox();
        channelsComboBox = new javax.swing.JComboBox();
        omitTrailingGapCheckBox = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        sampleSizeComboBox = new javax.swing.JComboBox();
        jLabel3 = new javax.swing.JLabel();

        sampleFrequencyComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "44100", "48000", "96000", "192000" }));
        sampleFrequencyComboBox.setSelectedIndex(1);
        sampleFrequencyComboBox.setToolTipText("Sample frequency of the generated signal.");

        channelsComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1", "2" }));
        channelsComboBox.setToolTipText("Number of channnels in generated signal. Normal is one channel. If two, the second channel is just exactly the opposite of the first.");

        omitTrailingGapCheckBox.setMnemonic('O');
        omitTrailingGapCheckBox.setText("Omit trailing gap");
        omitTrailingGapCheckBox.setToolTipText("If true, the last silence is left out of the generated wave signal.");

        jLabel1.setText("Sample freq.");

        jLabel2.setText("# Channels");

        sampleSizeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "8", "16" }));
        sampleSizeComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sampleSizeComboBoxActionPerformed(evt);
            }
        });

        jLabel3.setText("Bits");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(sampleFrequencyComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(channelsComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(9, 9, 9)
                        .addComponent(sampleSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(omitTrailingGapCheckBox))
                    .addGroup(layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel3)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2)
                    .addComponent(jLabel3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(sampleFrequencyComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(channelsComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(omitTrailingGapCheckBox)
                    .addComponent(sampleSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void sampleSizeComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sampleSizeComboBoxActionPerformed
        sampleSize = Integer.parseInt((String) sampleSizeComboBox.getSelectedItem());
    }//GEN-LAST:event_sampleSizeComboBoxActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox channelsComboBox;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JCheckBox omitTrailingGapCheckBox;
    private javax.swing.JComboBox sampleFrequencyComboBox;
    private javax.swing.JComboBox sampleSizeComboBox;
    // End of variables declaration//GEN-END:variables
}
